/* SPDX-License-Identifier: GPL-2.0-only OR MIT */
/*
 * Copyright 2025 NXP
 */

#ifndef DT_BINDING_RESET_IMX8MP_AUDIOMIX_H
#define DT_BINDING_RESET_IMX8MP_AUDIOMIX_H

#define IMX8MP_AUDIOMIX_EARC_RESET	0
#define IMX8MP_AUDIOMIX_EARC_PHY_RESET	1
#define IMX8MP_AUDIOMIX_DSP_RUNSTALL	2

#endif /* DT_BINDING_RESET_IMX8MP_AUDIOMIX_H */
