/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _ASM_X86_PURGATORY_H
#define _ASM_X86_PURGATORY_H

#ifndef __ASSEMBLER__
#include <linux/purgatory.h>

extern void purgatory(void);
#endif	/* __ASSEMBLER__ */

#endif /* _ASM_X86_PURGATORY_H */
