/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_CLOCK_ECONET_EN751221_SCU_H_
#define _DT_BINDINGS_CLOCK_ECONET_EN751221_SCU_H_

#define EN751221_CLK_PCIE	0
#define EN751221_CLK_SPI	1
#define EN751221_CLK_BUS	2
#define EN751221_CLK_CPU	3
#define EN751221_CLK_GSW	4

#endif /* _DT_BINDINGS_CLOCK_ECONET_EN751221_SCU_H_ */
