/* SPDX-License-Identifier: GPL-2.0+
 *
 * Copyright (C) 2015 Renesas Electronics Corp.
 */
#ifndef __DT_BINDINGS_CLOCK_RENESAS_CPG_MSSR_H__
#define __DT_BINDINGS_CLOCK_RENESAS_CPG_MSSR_H__

#define CPG_CORE			0	/* Core Clock */
#define CPG_MOD				1	/* Module Clock */

#endif /* __DT_BINDINGS_CLOCK_RENESAS_CPG_MSSR_H__ */
