/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_CLK_MT6735_VDECSYS_H
#define _DT_BINDINGS_CLK_MT6735_VDECSYS_H

#define CLK_VDEC_VDEC			0
#define CLK_VDEC_SMI_LARB1		1

#endif /* _DT_BINDINGS_CLK_MT6735_VDECSYS_H */
