/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (C) 2017 MediaTek Inc.
 */

#ifndef _DT_BINDINGS_POWER_MT7622_POWER_H
#define _DT_BINDINGS_POWER_MT7622_POWER_H

#define MT7622_POWER_DOMAIN_ETHSYS	0
#define MT7622_POWER_DOMAIN_HIF0	1
#define MT7622_POWER_DOMAIN_HIF1	2
#define MT7622_POWER_DOMAIN_WB		3
#define MT7622_POWER_DOMAIN_AUDIO	4

#endif /* _DT_BINDINGS_POWER_MT7622_POWER_H */
